#include "MakeRandomVector.hpp"
#include "benchmark/benchmark.h"
#include "oneapi/tbb.h"

#include <cmath>
#include <execution>
#include <numeric>

void BM_par_red_stl(benchmark::State &state) {
  const auto max_threads = state.range(0);
  const auto guard = oneapi::tbb::global_control{
      oneapi::tbb::global_control::max_allowed_parallelism, max_threads};
  const int n = 1 << 24;
  const auto input = makeRandomVector<double>(n);
  const auto range = oneapi::tbb::blocked_range<int>{0, n};
  for (auto _ : state) {
    double sum = std::transform_reduce(std::execution::par_unseq, input.begin(),
                                       input.end(), 0., std::plus{},
                                       [](double x) { return std::sin(x); });
    benchmark::DoNotOptimize(sum);
  }
  state.counters["Flops"] = benchmark::Counter{
      static_cast<double>(state.iterations()) * n, benchmark::Counter::kIsRate,
      benchmark::Counter::kIs1000};
}
BENCHMARK(BM_par_red_stl)->UseRealTime()->RangeMultiplier(2)->Range(1, 16);
BENCHMARK_MAIN();
